************************************************************************
*PROCEDURE DemoProcess
****************************
***  Function: Processes incoming Web Requests for DemoProcess
***            requests. This function is called from the wwServer 
***            process.
***      Pass: loServer -   wwServer object reference
*************************************************************************
LPARAMETER loServer
LOCAL loProcess
#INCLUDE WCONNECT.H

loProcess=CREATE("DemoProcess",loServer)
loProcess.lShowRequestData = loServer.lShowRequestData

IF VARTYPE(loProcess)#"O"
   *** All we can do is return...
   WAIT WINDOW NOWAIT "Unable to create Process object..."
   RETURN .F.
ENDIF

*** Call the Process Method that handles the request
loProcess.Process()

RETURN

*************************************************************
DEFINE CLASS DemoProcess AS WWC_PROCESS
*************************************************************

HTMLPath = ''
oAsync = Null

*********************************************************************
* Function DemoProcess :: Process
************************************
*** If you need to hook up generic functionality that occurs on
*** every hit, implement this method then call DoDefault() to
*** get the default Request Processing functionality. See docs
*** for more info on how to customize wwProcess::Process behavior.
*********************************************************************
FUNCTION Process
LOCAL lcDataDir, lnLoc1, lnLoc2, lcStartPath

ON ERROR 
*!*	#IF NOT DEBUGMODE
*!*		ON error Process.ERRLOG(lineno())
*!*	#ENDIF

this.InitSession()
this.HTMLPath = this.oServer.oConfig.oDemoProcess.cHTMLPagePath

if not This.Login()
	return .F.
endif

*!*	#IF NOT DEBUGMODE
*!*		ON error Process.ERRLOG(lineno(),this.oSession.GetSessionVar('UserName'))
*!*	#ENDIF

DODEFAULT()

RETURN .T.

ENDFUNC
*********************************************************************
* Function Login

*  Uses a VFP table for logins.
*  Users are tracked through the site by their e-mail address.  Uses a session cookie for keeping track of the
*  e-mail address.
FUNCTION Login
PUBLIC gcOwner
local lcUserName, lcOwner, lnLoc

Request  = this.oRequest
Response = this.oResponse
Session  = this.oSession

IF TYPE('Session') = 'O' and not empty(Session.GetSessionVar('UserName'))
	RETURN .T.
ENDIF 

if not This.OpenTable('Users')
	return .F.
endif

lcUserName = Request.Form("txtUserName")

IF EMPTY(lcUserName)
	This.ErrorMsg('No user name','You must log in',,3,"Login.htm")
	RETURN .F.
ENDIF
	
locate for UPPER(LoginName) = UPPER(lcUserName)
if eof()
	This.ErrorMsg('User Name not found', 'The user name you entered is not in our database.')
	return .F.
endif

gcOwner = OwnerName
Session.SetSessionVar('Owner',OwnerName)
Session.SetSessionVar('UserName',lcUserName)

This.DemoOptions()

RETURN .T.

ENDFUNC 
*---------------------------------------------------------------------------
FUNCTION DemoOptions

Response.clear()
Response.Writeln(this.StdHeader())
Response.Writeln([<center>])
Response.Writeln([<p><a href="Invoices.wc">Invoice Add/Edit/Delete</a></p>])
Response.Writeln([<p><a href="ShowOwner.ism">Public Variable</a></p>])
Response.Writeln([<p><a href="OneCursor.ism">Two users one cursor</a></p>])
Response.Writeln([<p><a href="ServerError.ism">Error on the server</a></p>])
Response.Writeln([<p><a href="StatusMessage.ism">Status message on the server</a></p>])
Response.Writeln([<p><a href="URLInvoices.wc">URL Hacking</a></p>])
Response.Writeln([<p><a href="FindOwners.htm">SQL Injection</a></p>])
Response.Writeln([</center>])
Response.Writeln(this.StdFooter())

ENDFUNC
*---------------------------------------------------------------------------
FUNCTION ShowOwner

* The wrong way:
* This command uses a public variable, which is changed when a user logs in.
* It will always be set to the owner for the last user logged in.
this.StandardPage("Show Me The Owner", ;
	"Current setting of Owner is <b>"+gcOwner+"</b>",,10,"DemoOptions.ism")

* The right way:
* Use a session variable instead.
*this.StandardPage("Show Me The Owner", ;
	"Current setting of Owner is <b>"+Session.GetSessionVar("Owner")+"</b>",,10,"DemoOptions.ism")

RETURN 
*---------------------------------------------------------------------------
FUNCTION OneCursor
LOCAL lcOwner

lcOwner = Session.GetSessionVar("Owner")
SELECT * ;
	from Invoices ;
	WHERE OwnerName = lcOwner ;
	INTO CURSOR InvTemp

this.ShowRecord()

ENDFUNC 
*---------------------------------------------------------------------------
FUNCTION ShowRecord

SELECT InvTemp
Response.Clear()
Response.Writeln(This.StdHeader())
Response.Writeln([<center>Owner: ]+OwnerName)
Response.Writeln([<br>Invoice#: ]+TRANSFORM(InvNo)+[  Date: ]+DTOC(InvDate))
Response.Writeln([<br>Total: ]+TRANSFORM(InvTotal)+[</center>])

* Write out the buttons at the bottom
Response.Writeln([<form method="post" action="NextPrevRecord.ism">])
Response.Writeln([<p><center><input type="submit" value="Previous" name="cmdSubmit">])
Response.Writeln([<input type="submit" value="Next" name="cmdSubmit">])
Response.Writeln([<input type="submit" value="Return to options" name="cmdSubmit"></center></p>])
Response.Writeln([</form>])

Response.Writeln(This.StdFooter())
                  
ENDFUNC
*---------------------------------------------------------------------------
FUNCTION NextPrevRecord
LOCAL lcWhichButton

lcWhichButton = Request.Form("cmdSubmit")
SELECT InvTemp
DO CASE 
CASE lcWhichButton = "Next"
	IF NOT EOF()
		SKIP 
		IF EOF()
			GOTO BOTTOM 
		ENDIF 
		this.ShowRecord()
	ENDIF 
CASE lcWhichButton = "Previous"
	IF NOT BOF()
		SKIP -1
		IF BOF()
			GOTO TOP 
		ENDIF 
		this.ShowRecord()
	ENDIF 
OTHERWISE
	this.DemoOptions()
ENDCASE 

ENDFUNC
*---------------------------------------------------------------------------
FUNCTION ServerError

* An error without any error handling:
X = Y

* A better way:
* You've got to return something to the browser, even if there's a error:
*!*	ON ERROR Process.ErrorMsg("Oops...", ;
*!*				"Looks like there was an error in the program.  Try something else.",,10,"DemoOptions.ism")
*!*	X = Y
*!*	ON ERROR 

* You don't want to QUIT like you may be doing in your LAN apps!

ENDFUNC 
*---------------------------------------------------------------------------
FUNCTION StatusMessage
LOCAL lnMilliseconds, llXit

* This technique displays the status message on the server, which the user will never see.
* Chances are, the user will give up, if their browser doesn't timeout first
DECLARE Sleep IN WIN32API integer lnMilliseconds
lnMilliseconds = 1000

llXit = .F.
ON ESCAPE llXit = .T.
SET ESCAPE ON 
FOR i = 1 TO 100
	IF llXit
		EXIT 
	ENDIF 

	WAIT WINDOW NOWAIT "I'm processing number "+TRANSFORM(i)
	sleep(lnMilliseconds)
ENDFOR 

IF NOT llXit
	this.StandardPage("Done","Here are your results")
ENDIF 

* Instead, you need to run an asynchronous process along these lines
* (SlowTasks.prg is a slightly modified version of SlowListTasks.prg, written by J. Randy Pearson for the WebRAD book)

*This.SlowTasks()

ENDFUNC
*---------------------------------------------------------------------------
FUNCTION ProcessInv
LOCAL lcButton

lcButton = UPPER(Request.Form("cmdSubmit"))
IF EMPTY(lcButton)
	lcButton = UPPER(Request.Form("cmdDelete"))
ENDIF 

do case
CASE lcButton = "ADD"
	this.AddInv()
CASE lcButton = "FIND"
 	this.FindInv()
CASE lcButton = "SAVE"
	this.SaveInv()
CASE lcButton = "CANCEL"
	RELEASE oInvoices
	Response.ExpandTemplate(ADDBS(This.HTMLPath)+"Invoices.wc")
CASE lcButton = "RETURN"
	This.DemoOptions()
CASE lcButton = "REPORT"
	this.DoReport()
OTHERWISE
	* We're don't get the value of the button when we use JavaScript to submit the form
	* Instead, we use a hidden field that is set by JavaScript when we choose the delete option
	IF UPPER(Request.Form('txtDelete')) = "DELETE"
		this.DelInv()
	ELSE
		this.DemoOptions()
	ENDIF 
ENDCASE

ENDFUNC 
*---------------------------------------------------------------------------
FUNCTION AddInv
LOCAL lcOwner

IF not this.OpenTable("Invoices")
	return
ENDIF

lcOwner = Session.GetSessionVar("Owner")
INSERT into Invoices (OwnerName) values (lcOwner)
PRIVATE oInvoices
SCATTER name oInvoices

* Here's one way to show the data--use a template
Response.ExpandTemplate(ADDBS(This.HTMLPath)+"Invoices.wc")
* Here's another way--write it out from VFP
*This.WriteRecord()

ENDFUNC
*---------------------------------------------------------------------------
FUNCTION FindInv
LOCAL lnInvNo, lcReturnPath

IF not this.OpenTable("Invoices")
	RETURN 
ENDIF 

lnInvNo = VAL(Request.Form('txtInvNo'))
IF EMPTY(lnInvNo)
	lnInvNo = VAL(Request.QueryString("InvNo"))
	lcReturnPath = "URLInvoices.wc"
ELSE
	lcReturnPath = "Invoices.wc"
ENDIF 

LOCATE for InvNo = lnInvNo
IF UPPER(OwnerName) <> UPPER(Session.GetSessionVar("Owner")) AND NOT EOF()
	this.StandardPage("Not your invoice","Sorry, but this invoice number doesn't belong to you",,5,"DemoOptions.ism")
ELSE 
	IF EOF()
		this.StandardPage("Locate error","I couldn't find invoice "+TRANSFORM(lnInvNo)+" and I'm not sure what to do about it",,5,"DemoOptions.ism")
	ELSE
		PRIVATE oInvoices
		SCATTER name oInvoices
		* Here's one way to show the data--use a template
		Response.ExpandTemplate(ADDBS(This.HTMLPath)+lcReturnPath)
		* Here's another way--write it out from VFP
*		This.WriteRecord()
	ENDIF 
ENDIF 

ENDFUNC 
*---------------------------------------------------------------------------
FUNCTION DelInv

IF not this.OpenTable("Invoices")
	RETURN 
ENDIF 

lnInvNo = VAL(Request.Form('txtInvNo'))
IF EMPTY(lnInvNo)
   lnInvNo = VAL(Request.QueryString("InvNo"))
ENDIF 

LOCATE for InvNo = lnInvNo
IF OwnerName <> Session.GetSessionVar("Owner") AND NOT EOF()
	this.ErrorMsg("Hacker Alert!","You hacked the URL, <br><b>you bad person, you!</b>",,15,"DemoOptions.ism")
ELSE 
	IF EOF()
		this.StandardPage("Locate error","I couldn't find invoice "+TRANSFORM(lnInvNo)+" and I'm not sure what to do about it",,5,"DemoOptions.ism")
	ELSE
		DELETE
		this.StandardPage("Deleted","Invoice "+TRANSFORM(lnInvNo)+" has been deleted!",,3,"DemoOptions.ism")
	ENDIF 
ENDIF 

ENDFUNC
*---------------------------------------------------------------------------
FUNCTION SaveInv
LOCAL lnInvNo, lcOwnerName, ldInvDate, lnInvTotal

lnInvNo     = VAL(Request.Form('txtInvNo'))
IF EMPTY(lnInvNo)
   lnInvNo = VAL(Request.QueryString("InvNo"))
ENDIF 

lcOwnerName = Request.Form('txtOwner')
ldInvDate   = CTOD(Request.Form('txtInvDate'))
lnInvTotal  = VAL(Request.Form('txtInvTotal'))

IF not this.OpenTable("Invoices")
	RETURN 
ENDIF 

LOCATE for InvNo = lnInvNo
IF UPPER(OwnerName) <> UPPER(Session.GetSessionVar("Owner")) AND NOT EOF()
	this.StandardPage("Not your invoice","Sorry, but this invoice number doesn't belong to you",,5,"DemoOptions.ism")
ELSE 
	IF EOF()
		this.StandardPage("Locate error","I couldn't find invoice "+TRANSFORM(lnInvNo)+" and I'm not sure what to do about it",,5,"DemoOptions.ism")
	ELSE
		replace InvDate with ldInvDate, ;
				Invtotal with lnInvTotal

		this.StandardPage("Changes Saved","Your changes are saved!",,3,"DemoOptions.ism")
	ENDIF 
ENDIF 

ENDFUNC 
*---------------------------------------------------------------------------
FUNCTION WriteRecord

WITH Response
	.Write([<html>])
	.Write([])
	.Write([<head>])
	.Write([<title>Invoices</title>])
	.Write([</head>])
	.Write([])
	.Write([<body>])
	.Write([])
	.Write([<script language=JavaScript>])
	.Write([])
	.Write([function DelConfirm()])
	.Write([{])
	.Write([  if (confirm("Delete this invoice?"))])
	.Write([     frmInvoice.submit();])
	.Write([  else])
	.Write([     alert("No invoice was deleted");])
	.Write([}])
	.Write([</script>])
	.Write([])
	.Write([<h1 align="center">Invoices</h1>])
	.Write([<center>])
	.Write([<form method="POST" action="ProcessInv.ism" name="frmInvoice">])
	.Write([  <p>Invoice#: <input type="text" name="txtInvNo" value="] + ;
			iif(TYPE("oInvoices")='O',TRANSFORM(oInvoices.InvNo),'0') + [" size="20"></p>])
	.Write([  <p>Owner: <input type="text" name="txtOwner" value="] + ;
			iif(TYPE("oInvoices")='O',oInvoices.OwnerName,'') + [" size="20"></p>])
	.Write([  <p>Date: <input type="text" name="txtInvDate" value="] + ;
			iif(TYPE("oInvoices")='O',DTOC(oInvoices.InvDate),'  /  /    ') + [" size="20"></p>])
	.Write([  <p>Total: <input type="text" name="txtInvTotal" value="] + ;
			iif(TYPE("oInvoices")='O',ALLTRIM(STR(oInvoices.InvTotal,10,2)),'0') + [" size="20"></p>])
	.Write([  <p><input type="submit" value="Add Invoice" name="cmdSubmit">])
	.Write([     <input type="submit" value="Find" name="cmdSubmit">])
	.Write([     <input type="button" value="Delete Invoice" name="cmdDelete" onclick="DelConfirm()">])
	.Write([     <input type="submit" value="Save" name="cmdSubmit">])
	.Write([     <input type="submit" value="Cancel" name="cmdSubmit">])
	.Write([     <input type="submit" value="Return to options" name="cmdSubmit"></p>])
	.Write([</form>])
	.Write([</center>])
	.Write([])
	.Write([</body>])
	.Write([])
	.Write([</html>])
ENDWITH

ENDFUNC 
*---------------------------------------------------------------------------
FUNCTION DoReport
LOCAL lcOwner

lcOwner = ALLTRIM(Session.GetSessionVar("Owner"))
SELECT * ;
	FROM Invoices ;
	WHERE OwnerName = lcOwner ;
	INTO TABLE (ALLTRIM(lcOwner)+"Invoices") ;
	ORDER BY InvNo
	
this.ShowRpt(1)

ENDFUNC 
*---------------------------------------------------------------------------
FUNCTION ShowRpt
LPARAMETERS tnRecord
LOCAL lcOwner, lcCmdSubmit

IF EMPTY(tnRecord)
	tnRecord = VAL(Request.Form("txtRecord"))
ENDIF 
lcOwner = Session.GetSessionVar("Owner")

IF NOT This.OpenTable(ALLTRIM(lcOwner)+"Invoices")
	this.ErrorMsg("Error opening table","Could not open invoices report table",,5,"DemoOptions.ism")
	RETURN 
ENDIF 

lcCmdSubmit = UPPER(Request.Form("cmdSubmit"))
DO case
CASE EMPTY(lcCmdSubmit) OR lcCmdSubmit = "NEXT"
	IF tnRecord > RECCOUNT()
		GOTO bottom
	ELSE 
		GOTO tnRecord
	ENDIF
CASE lcCmdSubmit = "PREVIOUS"
	tnRecord = tnRecord-41
	IF tnRecord-41 < 1
		tnRecord = 1
	ENDIF 
	GOTO tnRecord
CASE lcCmdSubmit = "RETURN"
	this.DemoOptions()
	return
ENDCASE

WITH Response
	.writeln(this.StdHeader())

	.writeln([<form method="POST" action="ShowRpt.ism">])
	.writeln([   <center>])
	.writeln([   <font size="3"><b>Invoices for Owner: ]+ALLTRIM(OwnerName)+[</b></font><p></p>])
	
	.writeln([   <table>])
	.writeln([   <th>Invoice#</th><th>Date</th><th>Total</th>])
	
	SCAN WHILE RECNO() <= tnRecord+20
		.writeln([      <tr>])
		.writeln([         <td>]+TRANSFORM(InvNo)+[</td><td>]+DTOC(InvDate)+[</td><td align="right">]+STR(InvTotal,10,2)+[</td>])
		.writeln([      </tr>])
	ENDSCAN 

	.writeln([   </table>])
	.writeln([   <input type="hidden" name="txtRecord" value="]+TRANSFORM(tnRecord+21)+[">])
	
	.writeln([   <p><input type="submit" name="cmdSubmit" value="Next">])
	.writeln([      <input type="submit" name="cmdSubmit" value="Previous">])
	.writeln([      <input type="submit" name="cmdSubmit" value="Return to Options"></p>])

	.writeln([   </center>])
	.writeln([</form>])

	.writeln(this.StdFooter())
ENDWITH 

ENDFUNC 
*---------------------------------------------------------------------------
FUNCTION SlowTasks		&& by J. Randy Pearson

SET PROCEDURE TO wwAsyncWebRequest ADDITIVE  && not yet set in WCONNECT.PRG
LOCAL loAsync, lcAction, lcJobId, lnRefreshPeriod, lnRetries, lnTry, lcMessage

loAsync = CREATEOBJECT( "wwAsyncWebRequest")
* See if this is a new request or polling for a current request:
lcAction = Request.QueryString( 'Action')
lcAction = IIF( EMPTY( m.lcAction), 'New', PROPER( m.lcAction))
IF m.lcAction == 'New'
	lcJobId = loAsync.SubmitEvent( , "Slow Task")
	* Optionally, fire up the process here. (Note: You
	* can also run a continuous background process that
	* polls for new requests--which is a better idea in a production environment.)
	DECLARE INTEGER ShellExecute IN SHELL32.DLL ;
	        INTEGER lnHWnd, ;
	        STRING lcAction, ;   
	        STRING lcFileName, ;
	        STRING lcExeParams, ;
	        STRING lcDefDir, ;
	        INTEGER lnShowWindow
	ShellExecute( 0, 'open', "AsyncDemo.exe", lcJobId, "", 1 )
ELSE  && not new
	lcJobId = Request.QueryString( 'JobId') 
ENDIF

* Set refresh periond (in seconds):
lnRefreshPeriod = 1
* How many polls before we give up:
lnRetries = 30
* Which retry are we on:
lnTry = 0
  
DO CASE
CASE m.lcAction == 'Check'
	LOCAL lnStatus
	lnStatus = loAsync.CheckForCompletion( m.lcJobId)
	* 0 means still running, negative numbers are bad news
	DO CASE
	CASE m.lnStatus = -2  && bad ID
		Process.ErrorMsg( "Invalid Request", ;
			"Sorry, your request was either invalid or has been lost. Please try again.",,5,"DemoOptions.ism" )
		this.oAsync = Null
		RETURN
	CASE m.lnStatus = -1  && cancel
		Process.ErrorMsg( "Request Cancelled","Your request has been cancelled.",,5,"DemoOptions.ism" )
		this.oAsync = Null
		RETURN
	CASE m.lnStatus = 1  && completed
		LOCAL lcResult
		lcResult = loAsync.oEvent.ReturnData

		This.StandardPage("Results",m.lcResult,,5,"DemoOptions.ism")
		this.oAsync = null
  
		RETURN
	ENDCASE
CASE m.lcAction == "Cancel"      
	loAsync.CancelEvent( m.lcJobId)
	Process.StandardPage( "Your request has been cancelled.",,,5,"DemoOptions.ism")
	this.oAsync = Null
	RETURN
ENDCASE

* If we get this far, we want to return a status update page:
lnTry = loAsync.oEvent.ChkCounter
IF m.lnTry > m.lnRetries
	Process.StandardPage( "Request Timeout", ;
		  "Sorry, but your request could not be completed in a timely fashion." + ;
		  " Please try again later.",,5,"DemoOptions.ism")
	this.oAsync = null 
	RETURN
ENDIF
  
lcMessage = 'Please wait. Your request is being processed. <br>' + ;
	[<table cellpadding=0 cellspacing=0 style="border: medium solid orange;" width=] + TRANSFORM( 5 * m.lnRetries + 2) + [>] + ;
	[<tr><td style="background-color: blue;"] + ;
	[ width=] + IIF(m.lnTry=0,"1",TRANSFORM( 5 * m.lnTry)) + ;
	[>&nbsp;</td><td>&nbsp;</td></tr></table><br>] + ;
	loAsync.oEvent.Status
lcMessage = m.lcMessage + [<p align=center>] + ;
	Response.HREF( 'SlowTasks.ism?action=Cancel&JobId=' + m.lcJobId, "Cancel", .T.)
 
This.StandardPage( "Asynchronous Task", m.lcMessage, , m.lnRefreshPeriod, ;
	'SlowTasks.ism?action=Check&JobId=' + m.lcJobId )

ENDFUNC
*---------------------------------------------------------------------------
FUNCTION SQLInjection
LOCAL lcOwner, lcOutput, lcWhere

lcOwner = UPPER(Request.Form("txtOwner"))

DO case
CASE LEN(lcOwner) > 10
	this.ErrorMsg("Invalid entry","Invalid length for owner")
	RETURN
CASE CHRTRAN(lcOwner,"<>][?~\!@#$%^&*()+=_'","") <> lcOwner
	this.ErrorMsg("Invalid entry","Invalid characters in owner")
	RETURN 
CASE CHRTRAN(lcOwner,["],[]) <> lcOwner
	this.ErrorMsg("Invalid entry","Invalid characters in owner")
ENDCASE 

IF EMPTY(lcOwner)
	lcWhere = []
ELSE
	lcWhere = "where UPPER(OwnerName) = [" + lcOwner + "]"
ENDIF

SELECT * ;
	FROM Invoices ;
	&lcWhere ;
	INTO CURSOR TmpInvoices

IF _tally > 0
	lcOutput = this.StdHeader()
	lcOutput = lcOutput + [<center>] + CRLF
	lcOutput = lcOutput + [<br>Invoices for ] + lcOwner + [:] + CRLF
	lcOutput = lcOutput + [<br><table>] + CRLF
	lcOutput = lcOutput + [<th>Invoice</th><th>Date</th><th>Amount</th>] + CRLF

	SCAN
		lcOutput = lcOutput + [<tr>] + CRLF
		lcOutput = lcOutput + [  <td>] + TRANSFORM(TmpInvoices.InvNo) + [</td>] + CRLF
		lcOutput = lcOutput + [  <td>] + DTOC(TmpInvoices.InvDate) + [</td>] + CRLF
		lcOutput = lcOutput + [  <td align="right">] + STR(TmpInvoices.InvTotal,10,2) + [</td>] + CRLF
		lcOutput = lcOutput + [</tr>] + CRLF
	ENDSCAN

	lcOutput = lcOutput + [</table>] + CRLF
	lcOutput = lcOutput + [</center>] + CRLF
	lcOutput = lcOutput + [<form method="POST" action="DemoOptions.ism">] + CRLF
	lcOutput = lcOutput + [<p align="center"><input type="submit" value="Done"></p>] + CRLF
	lcOutput = lcOutput + [</form>] + CRLF
	lcOutput = lcOutput + this.StdFooter()
	Response.Write(lcOutput)
ELSE
	this.StandardPage("No invoices found","There are no invoices for the owner entered",,5,"DemoOptions.ism")
ENDIF 

ENDFUNC
*---------------------------------------------------------------------------
FUNCTION StdHeader
LOCAL lcHeader

Response.Clear()
loHTTPHeader = CREATEOBJECT("wwHTTPHeader",Response)
loHTTPHeader.DefaultHeader()
loHTTPHeader.AddForceReload()
loHTTPHeader.AddCookie("laston",TTOC(DATETIME()))

lcHeader = loHTTPHeader.GetOutput()
lcHeader = lcHeader+[<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN">] + CRLF
lcHeader = lcHeader+[<html>] + CRLF
lcHeader = lcHeader+[<head>] + CRLF
lcHeader = lcHeader+[   <title>Logged in as ]+ALLTRIM(Session.GetSessionVar("Username"))+[</title>] + CRLF
lcHeader = lcHeader+[</head>] + CRLF
lcHeader = lcHeader+[<body>] + CRLF

RETURN lcHeader

ENDFUNC 
*********************************************************************
FUNCTION StdFooter
LOCAL lcFooter

lcFooter = [</body>]
lcFooter = lcFooter+[</html>]

RETURN lcFooter

ENDFUNC 
**********************************************************************
* Function OpenTable

*   Generic function for opening tables
*   Can be called regardless of whether or not the table was previously open.
*   Opens table named in tcTableName parameter.  If tcAlias is specified, table will be
*   opened with that alias.
*   Leaves table/alias selected
*   Returns .T. if successful, .F. if not
function OpenTable		&& (c) 2000, B. Peisch
lparameter tcTableName, tcAlias

local llSuccess, lcOldOnError

lcOldOnError = on('Error')
on error llSuccess = .F.

llSuccess = .T.
if empty(tcAlias)
	if not used(tcTableName)
		select 0
		use (tcTableName) again shared
	else
		select (tcTableName)
	endif
else
	if not used(tcAlias)
		select 0
		use (tcTableName) again alias (tcAlias) shared
	else
		select (tcAlias)
	endif
endif

on error &lcOldOnError

IF not llSuccess
	This.ErrorMsg('Table open error','Could not open '+tcTableName+' table')
ENDIF

return llSuccess

endfunc
*********************************************************************

ENDDEFINE